import tkinter as tk
from tkinter import filedialog, simpledialog, ttk, messagebox
from PIL import Image, ImageTk
import matplotlib.pyplot as plt
import numpy as np
from scipy.interpolate import griddata
import pywifi
from pywifi import const
import os
import time

class WiFiHeatmapApp:
    def __init__(self, root):
        self.root = root
        self.root.title("WiFi Heatmap Generator")
        
        self.frame = tk.Frame(root)
        self.frame.grid()
        # self.frame.pack(fill=tk.BOTH, expand=True)
        
        self.ssid_label = tk.Label(self.frame, text="Select SSID:")
        self.ssid_label.grid( row=0,column=1)
        # self.ssid_label.pack( pady=5)
        
        self.ssid_combobox = ttk.Combobox(self.frame)
        self.ssid_combobox.grid( row=0,column=2)
        # self.ssid_combobox.pack(pady=5)
        self.ssid_combobox.bind("<<ComboboxSelected>>", self.on_ssid_selected)

        self.map_button = tk.Button(self.frame, text="Update SSID", command=self.update_ssid_list)
        self.map_button.grid( row=0,column=3)
        # self.map_button.pack(pady=5)

        # self.rescan_button = tk.Button(self.frame, text="Rescan", command=self.value_init)
        # self.rescan_button.grid( row=0,column=4)

        self.map_button = tk.Button(self.frame, text="Select Map File", command=self.open_map)
        self.map_button.grid( row=0,column=5)
        # self.map_button.pack(pady=5)
        
        self.canvas = tk.Canvas(self.frame, width=800, height=600, bg='white')
        self.canvas.grid( row=1,column=0,rowspan=3,columnspan=8)
        # self.canvas.pack(fill=tk.BOTH, expand=True)
        self.canvas.bind("<ButtonPress-1>", self.On_drag_start)
        
        self.save_button = tk.Button(self.frame, text="Save Heatmap", command=self.save_heatmap)
        self.save_button.grid( row=10,column=2)
        # self.save_button.pack(pady=5)
        
        self.show_signal_var = tk.BooleanVar()
        self.show_signal_check = tk.Checkbutton(self.frame, text="Show Signal Levels", variable=self.show_signal_var)
        self.show_signal_check.grid( row=10,column=1)
        # self.show_signal_check.pack(pady=5)
        
        self.image = None
        self.image_path = None
        self.zoom_level = 1.0
        self.offset_x = 0
        self.offset_y = 0
        self.points = []
        self.signal_strengths = []
        
        self.canvas.bind("<Button-1>", self.on_click)
        self.canvas.bind("<MouseWheel>", self.on_zoom)
        
        self.wifi = pywifi.PyWiFi()
        self.iface = self.wifi.interfaces()[0]
        self.ssid_selected = False
        self.update_ssid_list()
        # self.root.after(2000, self.update_ssid_list_periodically)

    def value_init(self):
        self.image = None
        self.image_path = None
        self.zoom_level = 1.0
        self.offset_x = 0
        self.offset_y = 0
        self.points = []
        self.signal_strengths = []
        self.redraw_signals()
        
    def update_ssid_list_periodically(self):
        if not self.ssid_selected:
            self.update_ssid_list()
            self.root.after(2000, self.update_ssid_list_periodically)
        
    def update_ssid_list(self):
        self.iface.scan()
        time.sleep(1)  # Wait for the scan to complete
        results = self.iface.scan_results()
        ssids = list(set([network.ssid for network in results if network.ssid]))
        current_ssid = self.ssid_combobox.get()
        self.ssid_combobox['values'] = ssids
        if current_ssid in ssids:
            self.ssid_combobox.set(current_ssid)
        elif ssids:
            self.ssid_combobox.current(0)
    
    def on_ssid_selected(self, event):
        self.ssid_selected = True
    
    def open_map(self):
        self.image_path = filedialog.askopenfilename(filetypes=[("Image files", "*.jpg *.png *.jpeg *.bmp")])
        if self.image_path:
            self.image = Image.open(self.image_path)
            self.display_image()
    
    def display_image(self):
        if self.image:
            self.canvas.delete("all")
            width, height = self.image.size
            self.tk_image = ImageTk.PhotoImage(self.image.resize((int(width * self.zoom_level), int(height * self.zoom_level))))
            self.canvas.create_image(self.offset_x, self.offset_y, anchor=tk.NW, image=self.tk_image)
            self.redraw_signals()
    
    def redraw_signals(self):
        for (x, y), signal_strength in zip(self.points, self.signal_strengths):
            display_x = x * self.zoom_level
            display_y = y * self.zoom_level
            text_color = self.get_text_color(display_x, display_y)
            self.canvas.create_text(display_x, display_y, text=f"{signal_strength} dBm", fill=text_color)
    
    def on_click(self, event):
        x, y = event.x / self.zoom_level, event.y / self.zoom_level
        self.points.append((x, y))
        signal_strength = self.get_signal_strength()
        self.signal_strengths.append(signal_strength)
        text_color = self.get_text_color(event.x, event.y)
        self.canvas.create_text(event.x, event.y, text=f"{signal_strength} dBm", fill=text_color)
    
    def on_zoom(self, event):
        if event.delta > 0:
            self.zoom_level *= 1.1
        else:
            self.zoom_level /= 1.1
        self.display_image()
    
    def On_drag_start(self, event):
        self.drag_start_x = event.x
        self.drag_start_y = event.y

    def On_drag_end(self, event):
        self.drag_start_x = None
        self.drag_start_y = None

    def On_drag(self, event):
        if self.drag_start_x is not None and self.drag_start_y is not None:
            dx = event.x - self.drag_start_x
            dy = event.y - self.drag_start_y
            self.offset_x += dx
            self.offset_y += dy
            self.display_image()
            self.drag_start_x = event.x
            self.drag_start_y = event.y


    def get_signal_strength(self):
        ssid = self.ssid_combobox.get()
        self.iface.scan()
        time.sleep(0.5)  # Reduced wait time for faster response
        results = self.iface.scan_results()
        for network in results:
            if network.ssid == ssid:
                return network.signal
        return -100  # Default value if SSID not found
    
    def get_color(self, signal_strength):
        if signal_strength >= -50:
            return "green"
        elif signal_strength >= -60:
            return "yellowgreen"
        elif signal_strength >= -70:
            return "yellow"
        elif signal_strength >= -80:
            return "orange"
        else:
            return "red"
    
    def get_text_color(self, x, y):
        if self.image:
            img = self.image.resize((int(self.image.width * self.zoom_level), int(self.image.height * self.zoom_level)))
            pixel = img.getpixel((int(x), int(y)))
            r, g, b = pixel[:3]
            brightness = (r*299 + g*587 + b*114) / 1000
            if brightness < 128:
                return "white"
            else:
                return "black"
        return "black"
    
    def save_heatmap(self):
        if not self.points or not self.signal_strengths:
            return
        
        x, y = zip(*self.points)
        z = self.signal_strengths
        
        xi = np.linspace(min(x), max(x), 100)
        yi = np.linspace(min(y), max(y), 100)
        zi = griddata((x, y), z, (xi[None, :], yi[:, None]), method='cubic')
        
        fig, ax = plt.subplots()
        if self.image_path:
            img = plt.imread(self.image_path)
            img_height, img_width, _ = img.shape
            ax.imshow(img, extent=[0, img_width, img_height, 0], aspect='auto')
        
        heatmap = ax.imshow(zi, extent=(min(x), max(x), max(y), min(y)), origin='upper', cmap='RdYlGn', alpha=0.5)
        ax.scatter(x, y, c=z, cmap='RdYlGn', edgecolors='k')
        
        if self.show_signal_var.get():
            for (px, py), signal in zip(self.points, self.signal_strengths):
                text_color = self.get_text_color(px * self.zoom_level, py * self.zoom_level)
                ax.text(px, py, f"{signal} dBm", color=text_color, fontsize=8, ha='center')
        
        plt.colorbar(heatmap, ax=ax, label='Signal Strength (dBm)')
        
        save_path = filedialog.asksaveasfilename(defaultextension=".png", filetypes=[("PNG files", "*.png"), ("JPG files", "*.jpg"), ("PDF files", "*.pdf")])
        if save_path:
            plt.savefig(save_path)
            plt.close()
            messagebox.showinfo("Save Heatmap", "Heatmap has been saved successfully!")

if __name__ == "__main__":
    root = tk.Tk()
    app = WiFiHeatmapApp(root)
    root.mainloop()